@@ -12,12 +12,15 @@ import hashlib
 import hmac
 import json
 
+from yarl import URL
 from aiohttp import ClientConnectionError
 
 from mautrix.types import (UserID, FilterID, Filter, RoomEventFilter, RoomFilter, EventFilter,
-                           EventType, SyncToken, RoomID, Event, PresenceState, StateFilter)
+                           EventType, SyncToken, RoomID, Event, PresenceState, StateFilter,
+                           LoginType)
 from mautrix.appservice import AppService, IntentAPI
-from mautrix.errors import IntentError, MatrixError, MatrixRequestError, MatrixInvalidToken
+from mautrix.errors import (IntentError, MatrixError, MatrixRequestError, MatrixInvalidToken,
+                            WellKnownError)
 from mautrix.api import Path
 
 from .matrix import BaseMatrixHandler
@@ -37,12 +40,25 @@ class OnlyLoginSelf(CustomPuppetError):
         super().__init__("You may only replace your puppet with your own Matrix account.")
 
 
+class HomeserverURLNotFound(CustomPuppetError):
+    def __init__(self, domain: str):
+        super().__init__(f"Could not discover a valid homeserver URL for {domain}")
+
+
+class OnlyLoginTrustedDomain(CustomPuppetError):
+    def __init__(self):
+        super().__init__("This bridge doesn't allow double-puppeting "
+                         "with accounts on untrusted servers.")
+
+
 class CustomPuppetMixin(ABC):
     """
     Mixin for the Puppet class to enable Matrix puppeting.
 
     Attributes:
         sync_with_custom_puppets: Whether or not custom puppets should /sync
+        allow_discover_url: Allow logging into other homeservers using .well-known discovery.
+        homeserver_url_map: Static map from server name to URL that are always allowed to log in.
         only_handle_own_synced_events: Whether or not typing notifications and read receipts by
                                        other users should be filtered away before passing them to
                                        the Matrix event handler.
@@ -63,8 +79,10 @@ class CustomPuppetMixin(ABC):
     """
 
     sync_with_custom_puppets: bool = True
+    allow_discover_url: bool = False
+    homeserver_url_map: Dict[str, URL] = {}
     only_handle_own_synced_events: bool = True
-    login_shared_secret: Optional[bytes] = None
+    login_shared_secret_map: Dict[str, bytes] = {}
     login_device_name: Optional[str] = None
 
     az: AppService
@@ -78,6 +96,7 @@ class CustomPuppetMixin(ABC):
     default_mxid_intent: IntentAPI
     custom_mxid: Optional[UserID]
     access_token: Optional[str]
+    base_url: Optional[URL]
     next_batch: Optional[SyncToken]
 
     intent: IntentAPI
@@ -99,27 +118,33 @@ class CustomPuppetMixin(ABC):
         return bool(self.custom_mxid and self.access_token)
 
     def _fresh_intent(self) -> IntentAPI:
-        return (self.az.intent.user(self.custom_mxid, self.access_token)
+        return (self.az.intent.user(self.custom_mxid, self.access_token, self.base_url)
                 if self.is_real_user else self.default_mxid_intent)
 
     @classmethod
     def can_auto_login(cls, mxid: UserID) -> bool:
-        if not cls.login_shared_secret:
-            return False
         _, server = cls.az.intent.parse_user_id(mxid)
-        if server != cls.az.domain:
-            return False
-        return True
+        return server in cls.login_shared_secret_map and (server in cls.homeserver_url_map
+                                                          or server == cls.az.domain)
 
     @classmethod
     async def _login_with_shared_secret(cls, mxid: UserID) -> Optional[str]:
-        if not cls.can_auto_login(mxid):
+        _, server = cls.az.intent.parse_user_id(mxid)
+        try:
+            secret = cls.login_shared_secret_map[server]
+        except KeyError:
             return None
-        password = hmac.new(cls.login_shared_secret, mxid.encode("utf-8"),
-                            hashlib.sha512).hexdigest()
-        url = cls.az.intent.api.base_url + str(Path.login)
+        try:
+            base_url = cls.homeserver_url_map[server]
+        except KeyError:
+            if server == cls.az.domain:
+                base_url = cls.az.intent.api.base_url
+            else:
+                return None
+        password = hmac.new(secret, mxid.encode("utf-8"), hashlib.sha512).hexdigest()
+        url = base_url / str(Path.login)
         resp = await cls.az.http_session.post(url, data=json.dumps({
-            "type": "m.login.password",
+            "type": str(LoginType.PASSWORD),
             "initial_device_display_name": cls.login_device_name,
             "device_id": cls.login_device_name,
             "identifier": {
@@ -146,9 +171,29 @@ class CustomPuppetMixin(ABC):
             if not access_token:
                 raise ValueError("Failed to log in with shared secret")
             self.log.debug(f"Logged in for {mxid} using shared secret")
+
+        if mxid is not None:
+            _, mxid_domain = self.az.intent.parse_user_id(mxid)
+            if mxid_domain in self.homeserver_url_map:
+                base_url = self.homeserver_url_map[mxid_domain]
+            elif mxid_domain == self.az.domain:
+                base_url = None
+            else:
+                if not self.allow_discover_url:
+                    raise OnlyLoginTrustedDomain()
+                try:
+                    base_url = await IntentAPI.discover(mxid_domain, self.az.http_session)
+                except WellKnownError as e:
+                    raise HomeserverURLNotFound(mxid_domain) from e
+                if base_url is None:
+                    raise HomeserverURLNotFound(mxid_domain)
+        else:
+            base_url = None
+
         prev_mxid = self.custom_mxid
         self.custom_mxid = mxid
         self.access_token = access_token
+        self.base_url = base_url
         self.intent = self._fresh_intent()
 
         await self.start()
@@ -287,7 +332,7 @@ class CustomPuppetMixin(ABC):
         except asyncio.CancelledError:
             self.log.info(f"Syncing for {self.custom_mxid} cancelled")
         except Exception:
-            self.log.exception(f"Fatal error syncing {self.custom_mxid}")
+            self.log.critical(f"Fatal error syncing {self.custom_mxid}", exc_info=True)
 
     async def _sync(self) -> None:
         if not self.is_real_user:
@@ -303,7 +348,10 @@ class CustomPuppetMixin(ABC):
                 cur_batch = self.next_batch
                 sync_resp = await self.intent.sync(filter_id=filter_id, since=cur_batch,
                                                    set_presence=PresenceState.OFFLINE)
-                self.next_batch = sync_resp.get("next_batch", None)
+                try:
+                    self.next_batch = sync_resp.get("next_batch", None)
+                except Exception:
+                    self.log.warning("Failed to store next batch", exc_info=True)
                 errors = 0
                 if cur_batch is not None:
                     self._handle_sync(sync_resp)
