@@ -4,7 +4,7 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from typing import Dict, Any, Optional, List, TYPE_CHECKING
-from abc import ABC
+from abc import ABC, abstractmethod
 import logging
 import asyncio
 
@@ -13,6 +13,7 @@ from mautrix.appservice import AppService
 from mautrix.types import UserID, RoomID, EventType, Membership
 from mautrix.errors import MNotFound
 from mautrix.util.logging import TraceLogger
+from mautrix.util.opt_prometheus import Gauge
 
 from .portal import BasePortal
 
@@ -29,13 +30,16 @@ class BaseUser(ABC):
     loop: asyncio.AbstractEventLoop
 
     is_whitelisted: bool
+    is_admin: bool
     mxid: UserID
     command_status: Optional[Dict[str, Any]]
+    _metric_value: Dict[Gauge, bool]
 
     dm_update_lock: asyncio.Lock
 
+    @abstractmethod
     async def is_logged_in(self) -> bool:
-        return False
+        raise NotImplementedError()
 
     async def is_in_portal(self, portal: BasePortal) -> bool:
         try:
@@ -80,3 +84,11 @@ class BaseUser(ABC):
                 # Add DM statuses for all rooms in our database
                 current_dms.update(dms)
                 await puppet.intent.set_account_data(EventType.DIRECT, current_dms)
+
+    def _track_metric(self, metric: Gauge, value: bool) -> None:
+        if self._metric_value[metric] != value:
+            if value:
+                metric.inc(1)
+            else:
+                metric.dec(1)
+            self._metric_value[metric] = value
