@@ -12,3 +12,6 @@ from .crypto import (CryptoError, EncryptionError, SessionShareError, Decryption
                      MatchingSessionDecryptionError, DeviceValidationError, SessionNotFound,
                      DuplicateMessageIndex, VerificationError, DecryptedPayloadError,
                      MismatchingRoomError)
+from .well_known import (WellKnownError, WellKnownNotJSON, WellKnownMissingHomeserver,
+                         WellKnownNotURL, WellKnownUnexpectedStatus, WellKnownUnsupportedScheme,
+                         WellKnownInvalidVersionsResponse)
