@@ -3,7 +3,7 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Iterable, Awaitable, Optional, Type, Union, Tuple
+from typing import Iterable, Awaitable, Optional, Type, Union, Tuple, AsyncIterable, Any
 from itertools import chain
 from time import time
 import argparse
@@ -22,9 +22,14 @@ try:
 except ImportError:
     uvloop = None
 
+try:
+    import prometheus_client as prometheus
+except ImportError:
+    prometheus = None
 
-NewTask = Union[Awaitable[None], Iterable[Awaitable[None]]]
-TaskList = Iterable[Awaitable[None]]
+
+NewTask = Union[Awaitable[Any], Iterable[Awaitable[Any]], AsyncIterable[Awaitable[Any]]]
+TaskList = Iterable[Awaitable[Any]]
 
 
 class Program:
@@ -68,6 +73,7 @@ class Program:
             self.config_class = config_class
         self.startup_actions = []
         self.shutdown_actions = []
+        self._automatic_prometheus = True
 
     def run(self) -> None:
         """
@@ -160,6 +166,20 @@ class Program:
 
         self.loop = asyncio.get_event_loop()
 
+    def start_prometheus(self) -> None:
+        try:
+            enabled = self.config["metrics.enabled"]
+            listen_port = self.config["metrics.listen_port"]
+        except KeyError:
+            return
+        if not enabled:
+            return
+        elif not prometheus:
+            self.log.warning("Metrics are enabled in config, "
+                             "but prometheus_client is not installed")
+            return
+        prometheus.start_http_server(listen_port)
+
     def _run(self) -> None:
         signal.signal(signal.SIGINT, signal.default_int_handler)
         signal.signal(signal.SIGTERM, signal.default_int_handler)
@@ -190,7 +210,9 @@ class Program:
         First lifecycle method called inside the asyncio event loop. Extend this if you want more
         control over startup than just filling startup_actions in the prepare step.
         """
-        await asyncio.gather(*(self.startup_actions or []), loop=self.loop)
+        await asyncio.gather(*(self.startup_actions or []))
+        if self._automatic_prometheus:
+            self.start_prometheus()
 
     def prepare_stop(self) -> None:
         """
@@ -203,7 +225,7 @@ class Program:
         Lifecycle method used to stop things that need awaiting to stop. Extend this if you want
         more control over shutdown than just filling shutdown_actions in the prepare_stop method.
         """
-        await asyncio.gather(*(self.shutdown_actions or []), loop=self.loop)
+        await asyncio.gather(*(self.shutdown_actions or []))
 
     def prepare_shutdown(self) -> None:
         """Lifecycle method that is called right before ``sys.exit(0)``."""
@@ -216,12 +238,20 @@ class Program:
         self.startup_actions = self._add_actions(self.startup_actions, actions)
 
     def add_shutdown_actions(self, *actions: NewTask) -> None:
-        self.startup_actions = self._add_actions(self.startup_actions, actions)
+        self.shutdown_actions = self._add_actions(self.shutdown_actions, actions)
 
-    @staticmethod
-    def _add_actions(to: TaskList, add: Tuple[NewTask, ...]) -> TaskList:
+    async def _unpack_async_iterator(self, iterable: AsyncIterable[Awaitable[Any]]) -> None:
+        tasks = []
+        async for task in iterable:
+            if inspect.isawaitable(task):
+                tasks.append(asyncio.ensure_future(task, loop=self.loop))
+        await asyncio.gather(*tasks)
+
+    def _add_actions(self, to: TaskList, add: Tuple[NewTask, ...]) -> TaskList:
         for item in add:
-            if inspect.isawaitable(item):
+            if inspect.isasyncgen(item):
+                to.append(self._unpack_async_iterator(item))
+            elif inspect.isawaitable(item):
                 if isinstance(to, list):
                     to.append(item)
                 else:
